/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpeapplication.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include "Board.h"
#include "SpriteCollection.h"
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "kwin4proc.h"

Board::Board(QWidget *parent, const char *name, WFlags f) :
    QWidget(parent, name, f)
{
    m_sprites = new SpriteCollection(this);
    m_pixmap = new QPixmap(240, 240 + 22);
    m_game_running = false;
    m_movecount = 0;
    memset(m_move, 0, sizeof (m_move));
    m_state = PLAYER_MOVE;
    m_strength = 4;
    m_animations = true;
    m_computerBegins = false;
    memset(m_chip, 0, sizeof (m_chip));
    m_xsize = (240 - 40 - border * (cols - 1)) / cols;
    m_ysize = (240 - 22 - 40 - border * (rows - 1)) / rows;
    m_xoffs = 240 / 2 - (m_xsize * 7) / 2 - border * 3;
    m_yoffs = 240 / 2 - m_ysize * 3 - (border * 5) / 2 + 22 - 18;
}

Board::~Board()
{
    if (m_sprites) {
	delete m_sprites;
    }
    if (m_pixmap) {
	delete m_pixmap;
    }
}

void Board::paintEvent(QPaintEvent *event)
{
    if (!m_game_running) {
	start();
    } else {
	drawBoard(EMPTY, -1, -1);
    }
}

void Board::mousePressEvent(QMouseEvent *event)
{
    if (!m_game_running || (m_state != PLAYER_MOVE && m_state != HINT)) {
	return;
    }

    int row = (event->y() - m_yoffs + border / 2) / (m_ysize + border);
    int col = (event->x() - m_xoffs + border / 2) / (m_xsize + border);
    if (row < 0 || row >= rows || col < 0 || col >= cols) {
	return;
    }
    if (!checkMove(col, row)) {
	return;
    }

    if (m_state == HINT) {
	m_state = PLAYER_MOVE;
	repaint(false);
    } else {
	m_state = PLAYER_ANIM;
	repaint(false);
	animateMove(YELLOW, col, row);
	m_chip[row][col] = YELLOW;
	m_move[m_movecount++] = col;
	repaint(false);
	if (moveWon()) {
	    gameOver(YELLOW);
	    start();
	} else if (moveNotPossible()) {
	    gameOver(EMPTY);
	    start();
	} else {
	    computerMove();
	}
    }
}

void Board::start(void)
{
    m_game_running = false;
    m_state = m_computerBegins ? COMPUTER_MOVE : PLAYER_MOVE;
    memset(m_chip, 0, sizeof (m_chip));
    m_movecount = 0;
    memset(m_move, 0, sizeof (m_move));
    m_game_running = true;
    repaint(false);
    if (m_computerBegins) {
	computerMove();
    }
}

void Board::back(void)
{
    if (!m_game_running || m_state != PLAYER_MOVE || m_movecount < 2) {
	return;
    }
    backMove();
    backMove();
    repaint(false);
}

void Board::hint(void)
{
    if (!m_game_running || m_state != PLAYER_MOVE) {
	return;
    }

    int col = computeNextMove(YELLOW);
    if (col == -1) {
	return;
    }

    int x = m_xoffs + (m_xsize + border) * col;
    int i;
    m_state = HINT;
    repaint(false);
    for (i = 0; i < 240 && m_state == HINT; i++) {
	if (i % 2) {
	    drawBoard(EMPTY, x, m_yoffs);
	} else {
	    drawBoard(YELLOW, x, m_yoffs);
	}
	for (i = 0; i < 5; i++) {
	    usleep(50000);
	    qApp->processEvents();
	}
    }
    m_state = PLAYER_MOVE;
    repaint(false);
}

void Board::updateStrength(int strength)
{
    if (strength != m_strength) {
	m_strength = strength;
	if (m_game_running) {
	    start();
	}
    }
}

void Board::updateAnimations(int animations)
{
    if ((animations != 0) != m_animations) {
	m_animations = animations ? true : false;
    }
}

void Board::updateComputerBegins(int computerBegins)
{
    if ((computerBegins != 0) != m_computerBegins) {
	m_computerBegins = computerBegins ? true : false;
	if (m_game_running) {
	    start();
	}
    }
}

bool Board::checkMove(int &col, int &row)
{
    for (row = 0; row < (rows - 1); row++) {
	if (m_chip[row + 1][col] != EMPTY) {
	    break;
	}
    }
    return m_chip[0][col] == EMPTY;
}

void Board::animateMove(Chip who, int col, int row)
{
    if (!m_animations) {
	return;
    }

    int x = m_xoffs + (m_xsize + border) * col;
    int y_end = m_yoffs + (m_ysize + border) * row;
    int y_act = 22;

    while (y_act < y_end) {
	drawBoard(who, x, y_act);
	usleep(25000);
	y_act = y_act + 8;
	if (y_act >= y_end) {
	    y_act = y_end;
	    break;
	}
    }
}

void Board::computerMove(void)
{
    m_state = COMPUTER_MOVE;
    repaint(false);
    int col = computeNextMove(RED);
    int row = 0;
    if (col == -1 || !checkMove(col, row)) {
	m_state = PLAYER_MOVE;
	repaint(false);
	return;
    }
    m_state = COMPUTER_ANIM;
    repaint(false);
    animateMove(RED, col, row);
    m_chip[row][col] = RED;
    m_move[m_movecount++] = col;
    repaint(false);
    if (moveWon()) {
	gameOver(RED);
	start();
    } else if (moveNotPossible()) {
	gameOver(EMPTY);
	start();
    } else {
	m_state = PLAYER_MOVE;
	repaint(false);
    }
}

bool Board::findRowOfFour(int col, int row, int dcol, int drow)
{
    int who = EMPTY;
    int n = 0;

    while (col >= 0 && col < cols && row >= 0 && row < rows) {
	switch (m_chip[row][col]) {
	case EMPTY:
	    who = EMPTY;
	    n = 0;
	    break;
	case RED:
	    if (who != RED) {
		who = RED;
		n = 1;
	    } else if (++n >= 4) {
		return true;
	    }
	    break;
	case YELLOW:
	    if (who != YELLOW) {
		who = YELLOW;
		n = 1;
	    } else if (++n >= 4) {
		return true;
	    }
	    break;
	}
	col = col + dcol;
	row = row + drow;
    }
    return false;
}

bool Board::moveWon(void)
{
    int row;
    int col;

    for (col = 0; col < cols; col++) {
	if (findRowOfFour(col, rows - 1, 0, -1)) {
	    return true;
	}
    }
    for (row = rows - 1; row >= 0; row--) {
	if (findRowOfFour(0, row, 1, 0)) {
	    return true;
	}
    }
    for (row = 3; row < rows; row++) {
	if (findRowOfFour(0, row, 1, -1) || findRowOfFour(cols - 1, row, -1, -1)) {
	    return true;
	}
    }
    for (col = cols - 4; col >= 0; col--) {
	if (findRowOfFour(col, rows - 1, 1, -1)) {
	    return true;
	}
    }
    for (col = 3; col < cols; col++) {
	if (findRowOfFour(col, rows - 1, -1, -1)) {
	    return true;
	}
    }
    return false;
}

bool Board::moveNotPossible(void)
{
    int col;

    for (col = 0; col < cols; col++) {
	if (m_chip[0][col] == EMPTY) {
	    return false;
	}
    }
    return true;
}

void Board::gameOver(Chip who)
{
    switch (who) {
    case EMPTY:
	QMessageBox::information(this, tr("Win4"), tr("Game over\nNobody wins"));
	break;
    case RED:
	QMessageBox::information(this, tr("Win4"), tr("Game over\nYou loose"));
	break;
    case YELLOW:
	QMessageBox::information(this, tr("Win4"), tr("Game over\nYou win"));
	break;
    }
}

void Board::drawBoard(Chip who, int x, int y)
{
    QPainter painter_pixmap(m_pixmap);
    QPainter painter_widget(this);
    QString label("");
    QColor black(0, 0, 0);

    switch (m_state) {
    case PLAYER_MOVE:
	label = tr("Your turn");
	break;
    case PLAYER_ANIM:
    case COMPUTER_ANIM:
	label = tr("Moving coin...");
	break;
    case COMPUTER_MOVE:
	label = tr("Thinking...");
	break;
    case HINT:
	label = tr("Hint...");
	break;
    }
    painter_pixmap.fillRect(0, 0, 240, 240 + 22, backgroundColor());
    painter_pixmap.setPen(black);
    painter_pixmap.drawText(7, 0, width(), 22, AlignLeft | AlignVCenter, label);

    int row;
    int col;
    m_sprites->drawBoard(painter_pixmap, 0, 22);
    for (row = rows - 1; row >= 0; row--) {
	for (col = 0; col < cols; col++) {
	    switch (m_chip[row][col]) {
	    case EMPTY:
		m_sprites->drawEmpty(painter_pixmap, m_xoffs + (m_xsize + 4) * col,
				     m_yoffs + (m_ysize + 4) * row);
		break;
	    case RED:
		m_sprites->drawRed(painter_pixmap, m_xoffs + (m_xsize + 4) * col,
				   m_yoffs + (m_ysize + 4) * row);
		break;
	    case YELLOW:
		m_sprites->drawYellow(painter_pixmap, m_xoffs + (m_xsize + 4) * col,
				      m_yoffs + (m_ysize + 4) * row);
		break;
	    }
	}
    }

    if (who != EMPTY && x != -1 && y != -1) {
	if (who == RED) {
	    m_sprites->drawRed(painter_pixmap, x, y);
	} else {
	    m_sprites->drawYellow(painter_pixmap, x, y);
	}
    }

    painter_pixmap.flush();
    painter_widget.drawPixmap(0, 0, *m_pixmap);
}

void Board::backMove(void)
{
    int col = m_move[--m_movecount];
    int row;

    for (row = 0; row < rows; row++) {
	if (m_chip[row][col] != EMPTY) {
	    m_chip[row][col] = EMPTY;
	    break;
	}
    }
}

int Board::computeNextMove(Chip who)
{
    KComputer computer;
    time_t start;
    time_t end;
    int row;
    int col;
    int i;
    int j;
    int datafield[rows][cols];

    for (i = 0; i < rows; i++) {
	for (j = 0; j < cols; j++) {
	    switch (m_chip[rows - i - 1][j]) {
	    case EMPTY:
		datafield[i][j] = -1;
		break;
	    case RED:
		datafield[i][j] = 1;
		break;
	    case YELLOW:
		datafield[i][j] = 0;
		break;
	    }
	}
    }
    start = time(0);
    col = computer.think(m_movecount, who == YELLOW ? 0 : 1, m_computerBegins ? 1 : 0,
			 m_strength, datafield);
    row = 0;
    end = time(0);
    if ((end - start) < 2) {
	for (i = 0; i < 20; i++) {
	    usleep(50000);
	    qApp->processEvents();
	}
    }
    if (checkMove(col, row)) {
	return col;
    }
    for (i = 0; i < 100; i++) {
	col = rand() % cols;
	row = 0;
	if (checkMove(col, row)) {
	    return col;
	}
    }
    for (col = 0; col < cols; col++) {
	row = 0;
	if (checkMove(col, row)) {
	    return col;
	}
    }
    return -1;
}

